@@ -1,4 +1,4 @@
-segreg <- function(formula, data, subset, weights, na.action, family=lm, control=seg.control(), 
+segreg <- function(formula, data, subset, weights, na.action, family=lm, offset, control=seg.control(), 
     transf=NULL, contrasts=NULL, model=TRUE, x=FALSE, var.psi=TRUE, ...){
   ##### ====================================================================================
   #DA FARE:
@@ -118,7 +118,7 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
     #browser()
     
     mf <- match.call(expand.dots = FALSE)
-    m <- match(c("formula", "data", "subset", "weights",  "na.action"), names(mf), 0L) #"offset",
+    m <- match(c("formula", "data", "subset", "weights",  "na.action", "offset"), names(mf), 0L) #"offset",
     mf <- mf[c(1, m)]
     mf$drop.unused.levels <- TRUE
     mf[[1L]] <- as.name("model.frame")
@@ -204,6 +204,9 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
     #npsiList <-unlist(sapply(l,function(xx) attr(xx,"npsi")))
     npsiList <- lapply(l,function(xx) attr(xx,"npsi"))
     estList <- lapply(l,function(xx) attr(xx,"est"))
+    
+    #browser()
+    
     if(length(setdiff(drop(unlist(sapply(estList, function(.x){if(any(is.na(.x))) 0 else unique(.x)}))), c(0,1)))>0) stop(" 'est' should include 0's and 1's only")
     
     if(length(intersect(nomiCoefUNPEN, nomiPS))>0) stop("any segmented variable included as linear term too?")
@@ -302,13 +305,21 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       }
     }
     npsiList <- unlist(npsiList)
-      
+    
+    #browser()
+    
     if(!any(sapply(psiList,is.list))) psiList <- rep(psiList, repl)
     if(!any(sapply(estList,is.list))) estList <- rep(estList, repl)
     if(!any(sapply(RList,is.list))) RList <- rep(RList, repl)
+    if(!any(sapply(fixpsiList,is.list))) fixpsiList<- rep(fixpsiList, repl)
+    
+    
     nomiPS.orig <- rep(nomiPS.orig, repl)
     Bfix <- rep(Bfix, repl)
-    fixpsiList<- rep(fixpsiList, repl)
+    
+    
+    
+    
     while(any(sapply(B,is.list))){
         id.vc<-which((sapply(B, is.list)))[1]
         nc<-length(B[[id.vc]])
@@ -334,6 +345,10 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
         RList <- append(RList, RList[[id.vc]], after = id.vc-1)
         RList[[id.vc+nc]]<-NULL
         
+        fixpsiList <- append(fixpsiList, fixpsiList[[id.vc]], after = id.vc-1)
+        fixpsiList[[id.vc+nc]]<-NULL
+        
+        
         #se la lista contiene solo NULL, non funziona...
         #penMatrixList <- append(penMatrixList, penMatrixList[[id.vc]], after = id.vc-1)
         #penMatrixList[[id.vc+nc]]<-NULL
@@ -379,24 +394,28 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
     if(any(sapply(estList, is.list))) stop(" One or more 'est' components misspecified")
     
     npsiList1<-id.contrR <- rep(NA, length(B))
+    psiListE<-psiListQ<-psiList
     for(j in 1:length(B)){
         #K<- npsiList[j]
         K <- if(!is.na(npsiList[nomiSeg[j]])) npsiList[nomiSeg[j]] else npsiList[j]
         npsiList1[j]<- K
         if(any(is.na(psiList[[j]]))){ 
-          if(control$quant) {
-            psiList[[j]]<- quantile(B[[j]], prob= seq(0,1,l=K+2)[-c(1,K+2)], names=FALSE)
-          } else {
-            psiList[[j]]<- (min(B[[j]])+ diff(range(B[[j]]))*(1:K)/(K+1))
-          }
+          #if(control$quant) {
+            psiListQ[[j]]<- quantile(B[[j]], prob= seq(0,1,l=K+2)[-c(1,K+2)], names=FALSE)
+          #} else {
+            psiListE[[j]]<- (min(B[[j]])+ diff(range(B[[j]]))*(1:K)/(K+1))
+          #}
         } else {
           K<-npsiList1[j]<-length(psiList[[j]])
         }
         if(!is.null(fixpsiList[[j]])) {
           Bfix[[j]]<- sapply(sort(fixpsiList[[j]]), function(.x) (B[[j]]-.x)*(B[[j]]>.x))
-          colnames(Bfix[[j]])<- paste("U", 1:length(fixpsiList[[j]]),".fixed.",nomiPS.orig[j], sep="")
+          #colnames(Bfix[[j]])<- paste("U", 1:length(fixpsiList[[j]]),".fixed.",nomiPS.orig[j], sep="")
+          colnames(Bfix[[j]])<- paste("U", 1:length(fixpsiList[[j]]),".fixed.",nomiPS.ps.unlist[j], sep="")
+          
         }
       #se per qualche termine ci sono le matrici dei vincoli sulle slope
+        
         #browser()
         
         j.ok=match(nomiSeg[j], names(RList), nomatch=0)
@@ -419,8 +438,21 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
           }
     }
     
-      
-      #NB: Poiche' ora psiList include i veri numeri dei psi, i codici vanno rilanciati
+    #browser()
+    
+    if(control$quant) {
+      psiList<-psiListQ
+      initial <- unlist(psiListE)
+      PSI1<- matrix(initial, n, length(initial), byrow = TRUE)
+      } else {
+        psiList<-psiListE
+        initial <- unlist(psiListQ)
+        PSI1<- matrix(initial, n, length(initial), byrow = TRUE)
+      }
+    #Quindi PSI1 e' una matrice di valori inziali di psi.. Vanno usati da seg.lm.fit.boot nel caso in cui i primi PSI
+    #non portino ad adattare un modello
+    
+    #NB: Poiche' ora psiList include i veri numeri dei psi, i codici vanno rilanciati
       for(i in 1:length(B)) {
         nomiCoefPSI[[i]]<- paste(paste("psi",1:length(psiList[[i]]), sep=""), nomiPS.ps.unlist[i], sep=".") ##oppure sep=".psi"
         nomiTerminiSEG[[i]]<-rep(nomiPS.ps.unlist[i], length(psiList[[i]]))
@@ -465,7 +497,7 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       #colnames(Z)<- unlist(nomiCoefPEN)
       initial <- unlist(psiList)
       PSI <- matrix(initial, n, length(initial), byrow = TRUE)
-      
+      #PSI1 di riserva definito sopra.
       #browser()
       
       #NB la matrie del disegno X include in nomi "seg(x)" e non va bene perche' poi da problemi con i 
@@ -497,15 +529,22 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       #limZ <-matrix(sapply(1:length(npsii), function(.x) rep(limZ[,.x],npsii[.x])), nrow=2, byrow = FALSE)
       limZ <- do.call(cbind, lapply(1:length(npsii), function(.x) matrix(limZ[,.x],nrow=2,ncol=npsii[.x])))
       rangeZ <- do.call(cbind, lapply(1:length(npsii), function(.x) matrix(rangeSmooth[[.x]],nrow=2,ncol=npsii[.x])))
+      colnames(rangeZ)<-rep(names(npsii), npsii) #unlist(nomiTerminiSEG)
+      
+      #browser()
+      
       #rangeZ<- matrix(sapply(1:length(npsii), function(.x) rep(rangeSmooth[[.x]],npsii[.x])), nrow=2, byrow = FALSE)
       
       
       invXtX<-Xty<-NULL
+      
+      
       #browser()
       opz<-list(toll=toll,h=h,stop.if.error=stop.if.error,L0=NULL,visual=visual,it.max=it.max,nomiOK=unlist(nomiCoefU), usesegreg=TRUE,
                 fam=family, eta0=NULL, maxit.glm=maxit.glm, id.psi.group=id.psi.group, gap=gap, limZ=limZ, rangeZ=rangeZ,
                 conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step, tol.opt=control$tol.opt,
-                pow=pow, visualBoot=visualBoot, digits=digits, fc=fc, RList=RList, nomiSeg=nomiSeg, seed=control$seed, min.n=control$min.n)
+                pow=pow, visualBoot=visualBoot, digits=digits, fc=fc, RList=RList, nomiSeg=nomiSeg, 
+                seed=control$seed, min.n=control$min.n, PSI1=PSI1)
       
       #browser()
       
@@ -538,7 +577,8 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
           }
       } else {
         #browser()
-        if(fitter0=="lm"){
+
+          if(fitter0=="lm"){
 
           if(n.boot <= 0) {
             obj <- seg.lm.fit(Y, X, Z, PSI, weights, offs, opz)
@@ -564,6 +604,8 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
         }
       }
       
+      #browser()
+      
       if(!is.list(obj)){
         warning("Estimation failed. Too many breakpoints? Returning a (g)lm fit..", call. = FALSE)
         
@@ -585,13 +627,17 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
         return(invisible(NULL))
       }
       #
+
       id.psi.group<-obj$id.psi.group
       npsi.groups <- tapply(id.psi.group, id.psi.group, length)
       psi<-obj$psi
       psi.values<-if(n.boot<=0) obj$psi.values else obj$boot.restart
       U<-obj$U
       V<-obj$V
+      R=cor(cbind(U,V))
+      if(any(abs(R[col(R)>row(R)])>=.99)) stop("Not enought information for the fit: \n too many psi/small sample/replicated data", call.=FALSE)
       rangeZ<-obj$rangeZ
+      
       #browser()
       colnames(rangeZ) <- unlist(nomiTerminiSEG)
       it <- obj$it
@@ -601,6 +647,7 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       objU <- obj$obj
       #beta.c <- coef(objU)[paste("U", 1:ncol(U), sep = "")]
       beta.c <- coef(objU)[obj$idU]
+      
       #browser()
       
       if(any(id.contrR)) {
@@ -629,6 +676,10 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       se.psi<-rep(NA,k)
       if(fitter0=="lm"){
         objV <- if(id.noOW) lm.fit(x = cbind(X, U, Vxb), y = Y) else lm.wfit(x = cbind(X, U, Vxb), y = Y, w = weights, offset = offs)
+        if(any(is.na(objV$coefficients))) {
+          warning("some interval with 1 observation. St.errs cannot be computed")
+          var.psi=FALSE
+        }
         if(var.psi) {
           s2 <- sum(weights*objU$residuals^2)/objV$df.residual
           R <- chol2inv(objV$qr$qr)
@@ -658,9 +709,10 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
           if (!obj0$converged) warning("fitting to calculate the null deviance did not converge -- increase 'maxit'?")
           objV$null.deviance <- obj0$deviance
         }
-        
-        
-        
+        if(any(is.na(objV$coefficients))) {
+          warning("some interval with 1 observation. St.errs cannot be computed")
+          var.psi=FALSE
+        }
         if(var.psi) {
           R <- chol2inv(objV$qr$qr)
           s2 <- 1
@@ -781,6 +833,7 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       #il comando structure() l'ho messo perche' avevo bisogno che anche in mancanza di offset, l'oggetto finale restituisse 
       # un oggetto con la componente offset NULL. Cosa che non viene fatta con il semplice comando di sotto.. 
       #objV<- if(id.O) c(objV, offset=offs) else c(objV, offset=NULL)
+      #browser()
       objV<- structure(c(objV, list(offset=orig.offs)))
       class(objV)<-c("segmented", class0)
       objV
